    
import unittest
from datetime import datetime
from unittest.mock import patch

import sys
import os
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from JGTPDSP import create_filestore_path


class TestCreateFilestorePath(unittest.TestCase):
    @patch("JGTPDSP.jgtos.create_filestore_path")
    def test_create_filestore_path(self, mock_create_filestore_path):
        # Arrange
        mock_create_filestore_path.return_value = "/tmp/data/EUR-USD_H1_2101010000_2101012359.csv"
        instrument = "EUR/USD"
        timeframe = "H1"
        quiet = True
        compressed = False
        tlid_range = "2101010000_2101012359"
        output_path="/tmp/data"
        # Act
        result = create_filestore_path(
            instrument, timeframe, quiet, compressed, tlid_range,output_path=output_path
        )
        print(result)
        # Assert
        mock_create_filestore_path.assert_called_once_with(
            instrument, timeframe, quiet, compressed, tlid_range,output_path
        )
        self.assertEqual(result, "/tmp/data/EUR-USD_H1_2101010000_2101012359.csv")


from JGTPDSP import mk_fn_range, mk_fn

class TestMkFnRange(unittest.TestCase):
    def test_mk_fn_range(self):
        # Arrange
        instrument = "EUR/USD"
        timeframe = "H1"
        start = datetime(2022, 1, 1)
        end = datetime(2022, 1, 2)
        ext = "csv"

        # Act
        result = mk_fn_range(instrument, timeframe, start, end, ext)

        # Assert
        expected_result = "EUR-USD_H1_2201010000_2201020000.csv"
        self.assertEqual(result, expected_result)


if __name__ == "__main__":
    unittest.main()





#!/usr/bin/env python3
import sys
import os
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

# import .

import jgtconstants as constants
import jgtcommon as jgtcommon
import argparse
import jgtwslhelper as wsl
import JGTPDSP as pds
import JGTCDS as cds

import pandas as pd

def parse_args():
    parser = argparse.ArgumentParser(description='Process command parameters.')
    #jgtfxcommon.add_main_arguments(parser)
    jgtcommon.add_instrument_timeframe_arguments(parser)
    jgtcommon.add_date_arguments(parser)
    jgtcommon.add_tlid_range_argument(parser)
    jgtcommon.add_max_bars_arguments(parser)
    #jgtcommon.add_output_argument(parser)
    #jgtfxcommon.add_quiet_argument(parser)
    jgtcommon.add_verbose_argument(parser)
    jgtcommon.add_cds_argument(parser)
    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    instrument = args.instrument
    timeframe = args.timeframe
    quotes_count = args.quotescount
    date_from = None
    date_to = None
    tlid_range = None
    if args.tlidrange:
        #@STCGoal Get range prices from cache or request new
        tlid_range = args.tlidrange
        print("FUTURE Support for tlid range")
        tmpcmd = wsl._mkbash_cmd_string_jgtfxcli_range(instrument,timeframe,tlid_range)
        print("> " + tmpcmd)
        print("-----------Stay tune -------- Quitting for now")
        return

    if args.datefrom:
        date_from = args.datefrom.replace('/', '.')
    if args.dateto:
        date_to = args.dateto.replace('/', '.')

    process_cds=args.cds
    #output=False
    #compress=False
    verbose_level = args.verbose
    quiet=False
    if verbose_level == 0:
        quiet=True
    #print("Verbose level : " + str(verbose_level))
    if process_cds:
        print("Processing CDS")
        output=True
    #if args.compress:
    #    compress = args.compress
    #     output = True # in case
    # if args.output:
    #     output = True

    if verbose_level > 1:
        if date_from:
            print("Date from : " + str(date_from))
        if date_to:
            print("Date to : " + str(date_to))


    try:
        
        print_quiet(quiet,"Getting for : " + instrument + "_" + timeframe)
        instruments = instrument.split(',')
        timeframes = timeframe.split(',')


        for instrument in instruments:
            for timeframe in timeframes:
                createCDS_for_main(instrument, timeframe, quiet, verbose_level,tlid_range)
                # else:
                #     p = pds.getPH(instrument, timeframe, quotes_count, date_from, date_to, False, quiet)
                #     if verbose_level > 0:
                #         print(p)
         
    except Exception as e:
        jgtcommon.print_exception(e)

    #try:
    #    jgtpy.off()
    #except Exception as e:
    #    jgtfxcommon.print_exception(e)

# if __name__ == "__main__":
#     main()

# print("")
# #input("Done! Press enter key to exit\n")

def createCDS_for_main(instrument, timeframe, quiet, verbose_level=0,tlid_range=None):
    # implementation goes here
    col2remove=constants.columns_to_remove
    config = jgtcommon.readconfig()
    if 'columns_to_remove' in config:  # read it from config otherwise
        col2remove = config['columns_to_remove']
    quietting=True
    if verbose_level> 1:
        quietting=False
    try: 
        cdspath,c=cds.createFromPDSFileToCDSFile(instrument,timeframe,col2remove,quietting)
        print_quiet(quiet,cdspath)
        print_quiet(quiet,c)
    except Exception as e:
        print("Failed to create CDS for : " + instrument + "_" + timeframe)
        print("Exception: " + str(e))



def print_quiet(quiet,content):
    if not quiet:
        print(content)



import os
import datetime

import sys
import os
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

import jgtcommon


def create_filestore_path(
    instrument, timeframe, quiet=True, compressed=False, tlid_range=None,output_path=None
):
    # Define the file path based on the environment variable or local path
    if output_path is None:
        data_path = get_data_path()
    else: # get path from var in os
        data_path = output_path
        
    if not quiet:
        print(data_path)

    ext = "csv"
    if compressed:
        ext = "csv.gz"

    fpath = mk_fullpath(instrument, timeframe, ext, data_path, tlid_range=tlid_range)

    if os.name == "nt":
        fpath = fpath.replace("/", "\\")
    return fpath


def mk_fn(instrument, timeframe, ext="csv"):
    """Make a file name with instrument and timeframe

    Args:
        instrument (str): symbol
        timeframe (str): TF
        ext (str): ext name "csv"

    Returns:
        str: file name
    """
    _tf = timeframe
    _i = instrument.replace("/", "-")
    if timeframe == "m1":
        _tf = timeframe.replace("m1", "mi1")
    _fn = _i + "_" + _tf + "." + ext
    return _fn.replace("..", ".")


def mk_fn_range(instrument, timeframe, start: datetime, end: datetime, ext="csv"):
    _tf = timeframe
    _i = instrument.replace("/", "-")
    if timeframe == "m1":
        _tf = timeframe.replace("m1", "mi1")
    start_str = jgtcommon.tlid_dt_to_string(start)
    end_str = jgtcommon.tlid_dt_to_string(end)
    _fn = f"{_i}_{_tf}_{start_str}_{end_str}.{ext}"
    # _fn= _i + '_' + _tf + '.' + ext
    _fn = _fn.replace("..", ".")
    _fn = _fn.replace("/", "-")
    return _fn


def mk_fullpath(instrument, timeframe, ext, path, tlid_range=None):
    if tlid_range is None:
        fn = mk_fn(instrument, timeframe, ext)
    else:
        start_dt, end_dt = jgtcommon.tlid_range_to_start_end_datetime(
            tlid_range=tlid_range
        )
        # print(str(start_dt),str(end_dt))
        fn = mk_fn_range(instrument, timeframe, start_dt, end_dt, ext)
    rpath = os.path.join(path, fn)
    if os.name == "nt":
        rpath = rpath.replace("/", "\\")
    # path + '/'+fn
    return rpath


# .replace('..','.').replace('//','/')


def get_data_path():
    default_dot = "./data"
    default_dotdot = "../data"
    data_path = os.environ.get("JGTPY_DATA", default_dot)

    if not os.path.exists(data_path):
        data_path = os.environ.get("JGTPY_DATA", default_dotdot)

    if os.name == "nt":
        data_path = data_path.replace("/", "\\")

    if not os.path.exists(data_path):
        raise Exception(
            "Data directory not found. Please create a directory named 'data' in the current or parent directory, or set the JGTPY_DATA environment variable."
        )

    data_path = os.path.join(data_path, "pds")
    return data_path



# Copyright 2023 Jean Guillaume Isabelle
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
.. moduleauthor:: Jean Guillaume Isabelle <jgi@jgwill.com>
"""


import os
import platform
import sys

import sys
import os
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))


import jgtflags


import warnings

with warnings.catch_warnings():
    warnings.filterwarnings("ignore", category=RuntimeWarning, module="importlib._bootstrap")
    # your code here

class NotCompatibleException(Exception):
    pass

#from jgtpy.common_samples import common_samples
from jgtetl import svc_offset_dt_by_tf as etl_offset_dt_by_tf,offsetdt as etl_offsetdt
from JGTCore import __version__ #,json2dict,jsonfile2prop,json2prop,jsonfile2dict,d2p,fixdtindf,offsetdt
#from .JGTConfig import getenv,setreal,setdemo,env



import jgtwslhelper as wsl

from JGTIDS import tocds as tocds
#mk_fn,mk_fullpath,getSubscribed,getPH,getPHByRange,tryConnect
from JGTCDS import create as createCDS,createByRange,createFromDF,getLast,getPresentBar,getPresentBarAsList,getLastCompletedBarAsList,createFromDF,createFromFile_and_clean_and_save_data as fromfile,createFromFile_and_clean_and_save_data as ff,createFromDF as fp

def help():
    print(".h(i,t,400)\t\tGet Prices (PDS): \n\t\t\tjgtpy.h(instrument,timeframe,quote_count=335,start=None,end=None,quiet=True)")
    print("\t\t\t\t\treturn DataFrame of PDS Type")
    print("\t\t\t\t\t(Will connect and stay connected)")
    print(".fp(df)\t\t Create CDS from PDS DF \n\t\t\tjgtpy.fp(df [pd.DataFrame])")
    print("\t\t\t\t\treturn DataFrame of CDS Type")
    print(".off()\t\t Disconnect (Bugged)\n\t\t\tjgtpy.off()")





#@title ADS

#%% Imports
import numpy as np
import pandas as pd
import warnings
import matplotlib.pyplot as plt
import mplfinance as mpf


import sys
import os
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

#import jgtpy
import JGTPDSP as pds 
import JGTIDS as ids
from JGTIDS import getMinByTF
import JGTCDS as cds
import jgtwslhelper as wsl

warnings.filterwarnings("ignore", category=FutureWarning)

import os

#import kaleido
import plotly

import JGTConfig as jgtc


import plotly.graph_objects as go
import plotly.subplots as sp



#%% Logging

import logging
_loglevel= logging.WARNING

# Create a logger object
l = logging.getLogger()
l.setLevel(_loglevel)

# Create a console handler and set its level
console_handler = logging.StreamHandler()
console_handler.setLevel(_loglevel)

# Create a formatter and add it to the console handler
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
console_handler.setFormatter(formatter)

# Add the console handler to the logger
l.addHandler(console_handler)




cdtformat="%Y-%m-%d"



#%% Props and requests

import adshelper as ah
from jgtpy import jgtconstants as c




def jgtxplot18c_231209(instrument,timeframe,nb_bar_on_chart = 375,recreate_data = True,show_plot=True):
    data = ah.prepare_cds_for_ads_data(instrument, timeframe, nb_bar_on_chart, recreate_data)
    return plot_from_cds_df(data,instrument,timeframe,nb_bar_on_chart,show_plot)


def plot_from_pds_df(pdata,instrument,timeframe,nb_bar_on_chart = 375,show_plot=True):
  # Select the last 400 bars of the data
  try:
      selected = pdata.iloc[-nb_bar_on_chart-120:].copy()
  except:
      selected = pdata.copy()
      l.warning("Could not select the desired amount of bars, trying anyway with what we have")
      pass
  
  data = cds.createFromDF(selected)
  return plot_from_cds_df(data,instrument,timeframe,nb_bar_on_chart,show_plot)
  
  
def plot_from_cds_df(data,instrument,timeframe,nb_bar_on_chart = 375,show_plot=True):
  # Load dataset
  iprop = pds.get_instrument_properties(instrument)
  l.debug(iprop)
  pipsize = iprop["pipsize"]  # Access 'pipsize' using dictionary-like syntax
  l.debug(pipsize)

  # Convert the index to datetime
  try:
      data.index = pd.to_datetime(data.index)
  except:
      l.error("Error converting index to datetime")
      pass



  main_plot_type="ohlc"

  _ao_coln = c.indicator_AO_awesomeOscillator_column_name
  _ac_coln = c.indicator_AC_accelerationDeceleration_column_name
  
  
  ao_upbar_color = "g"
  ao_dnbar_color = "r"
  ac_up_color = "darkgreen"
  ac_dn_color = "darkred"
  fdb_signal_buy_color = "g"
  fdb_signal_sell_color = "r"
  jaw_color = "blue"
  teeth_color = "red"
  lips_color = "green"
  fractal_up_color = "blue"
  fractal_dn_color = "blue"
  fractal_dn_color_higher = "blue"
  fractal_up_color_higher = "blue"
  ac_signal_buy_color = "lightgreen"
  ac_signal_sell_color = "yellow"

  fdb_marker_size = 7
  fractal_marker_size = 8
  ac_signals_marker_size = 24
  saucer_marker_size = 48
  fractal_degreehigher_marker_size = 20
  
  fdb_signal_marker = "o"
  fractal_up_marker="^"
  fractal_up_marker_higher= "^"
  fractal_dn_marker_higher = "v"
  fractal_dn_marker = "v"
  ac_signal_marker="o"

  #COLUMNS
  _jaw_coln = c.indicator_currentDegree_alligator_jaw_column_name
  _teeth_coln = c.indicator_currentDegree_alligator_teeth_column_name
  _lips_coln = c.indicator_currentDegree_alligator_lips_column_name

  _open_coln = c.open_column_name
  _high_coln = c.high_column_name
  _low_coln = c.low_column_name
  _close_coln = c.close_column_name
  _barheight_coln = "bar_height"

  fh_col_dim = c.indicator_fractal_high_degree2_column_name
  fl_col_dim = c.indicator_fractal_low_degree2_column_name
  fh_col_dim_higher = c.indicator_fractal_high_degree8_column_name
  fl_col_dim_higher = c.indicator_fractal_low_degree8_column_name


  _fdb_coln = c.signalCode_fractalDivergentBar_column_name
  _fdbb_coln = c.signalBuy_fractalDivergentBar_column_name
  _fdbs_coln = c.signalSell_fractalDivergentBar_column_name


  _acb_coln = c.signalBuy_AC_acceleration_column_name  
  _acs_coln = c.signalSell_AC_deceleration_column_name

  #plot config
  main_plot_panel_id=0
  ao_plot_panel_id=1
  ac_plot_panel_id=2

  plot_style = "yahoo"
  fig_ratio_x = 24
  fig_ratio_y = 10
  
  #%% Select the last 400 bars of the data

  # Select the last 400 bars of the data
  try:
      data_last_selection = data.iloc[-nb_bar_on_chart:].copy()
  except:
      l.warning("Could not select the desired amount of bars, trying anyway with what we have")
      data_last_selection = data
      pass

  # Make OHLC bars plot
  ohlc = data_last_selection[[_open_coln, _high_coln, _low_coln, _close_coln]]





  #%% AO/AC
  # AO / AC
  # Calculate the color for 'ao' and 'ac' bar
  colors_ao = [
      ao_upbar_color if (data_last_selection[_ao_coln][i] - data_last_selection[_ao_coln][i - 1] > 0) else ao_dnbar_color
      for i in range(1, len(data_last_selection[_ao_coln]))
  ]
  colors_ao.insert(0, ao_dnbar_color)

  colors_ac = [
      ac_up_color
      if (data_last_selection[_ac_coln][i] - data_last_selection[_ac_coln][i - 1] > 0)
      else ac_dn_color
      for i in range(1, len(data_last_selection[_ac_coln]))
  ]
  colors_ac.insert(0, ac_dn_color)
  # Make 'ao' and 'ac' oscillator plot
  ao_plot = mpf.make_addplot(
      data_last_selection[_ao_coln], panel=ao_plot_panel_id, color=colors_ao, secondary_y=False, type="bar"
  )
  ac_plot = mpf.make_addplot(
      data_last_selection[_ac_coln], panel=ac_plot_panel_id, color=colors_ac, secondary_y=False, type="bar"
  )
  # @STCGoal Make AO/AC signals plotted



  #%% Alligator

  #@STCIssue no offset data in, not offset columns: jaws_tmp,teeth_tmp,lips_tmp

  # Make Alligator's lines plot

  jaw_plot = mpf.make_addplot(data_last_selection[_jaw_coln], panel=main_plot_panel_id, color=jaw_color)
  teeth_plot = mpf.make_addplot(data_last_selection[_teeth_coln], panel=main_plot_panel_id, color=teeth_color)
  lips_plot = mpf.make_addplot(data_last_selection[_lips_coln], panel=main_plot_panel_id, color=lips_color)


  #%% Offsets and various values
  # offset

  min_timeframe = getMinByTF(timeframe)
  price_mean = data_last_selection[_close_coln].mean()

  # Calculate the bar height for each row
  data_last_selection[_barheight_coln] = data_last_selection[_high_coln] - data_last_selection[_low_coln]


  # Calculate the average bar height
  average_bar_height = data_last_selection[_barheight_coln].mean()
  low_min = data_last_selection[_low_coln].min()
  high_max = data_last_selection[_high_coln].max()
  ao_max = data_last_selection[_ao_coln].max()
  ao_min = data_last_selection[_ao_coln].min()
  ac_max = data_last_selection[_ac_coln].max()
  ac_min = data_last_selection[_ac_coln].min()


  #%% FDB

  # Align fdbb with OHLC bars if value is '1.0'
  data_last_selection.loc[:,_fdbb_coln] = np.where(
      data_last_selection[_fdb_coln] == 1.0, data_last_selection[_high_coln], np.nan
  )
  data_last_selection.loc[:,_fdbs_coln] = np.where(
      data_last_selection[_fdb_coln] == -1.0, data_last_selection[_low_coln], np.nan
  )

  # Date,Volume,Open,High,Low,Close,Median,ac,jaw,teeth,lips,bjaw,bteeth,blips,ao,fh,fl,fh3,fl3,fh5,fl5,fh8,fl8,fh13,fl13,fh21,fl21,fh34,fl34,fh55,fl55,fh89,fl89,fdbb,fdbs,fdb,aof,aoaz,aobz,zlc,zlcb,zlcs,zcol,sz,bz,acs,acb,ss,sb



  #%% Saucer
  _saucer_b_coln = c.signalBuy_saucer_column_name
  _saucer_s_coln = c.signalSell_saucer_column_name

  # Saucer
  # Align saucer with AO bars if value is '1.0'
  data_last_selection.loc[:,_saucer_b_coln] = np.where(data_last_selection[_saucer_b_coln] == 1.0, ao_max, np.nan)
  data_last_selection.loc[:,_saucer_s_coln] = np.where(data_last_selection[_saucer_s_coln] == 1.0, ao_min, np.nan)

  saucer_offset_value = 0
  
  # Make Buy Signal plot
  sb_plot = mpf.make_addplot(
      data_last_selection[_saucer_b_coln] + saucer_offset_value,
      panel=ao_plot_panel_id,
      type="scatter",
      markersize=saucer_marker_size,
      marker="|",
      color="g",
  )
  
  # Make Sell Signal plot
  ss_plot = mpf.make_addplot(
      data_last_selection[_saucer_s_coln] - saucer_offset_value,
      panel=ao_plot_panel_id,
      type="scatter",
      markersize=saucer_marker_size,
      marker="|",
      color="r",
  )

  #%% Outputs checks
  l.debug("---------------------------------------")
  l.debug("AC Min/Max: " + str(ac_min) + "/" + str(ac_max))
  l.debug("---------------------------------------")


  #%% AC Signal


  # Align saucer with OHLC bars if value is '1.0'
  data_last_selection.loc[:,_acb_coln] = np.where(
      data_last_selection[_acb_coln] == 1.0, data_last_selection[_ac_coln], np.nan
  )
  data_last_selection.loc[:,_acs_coln] = np.where(
      data_last_selection[_acs_coln] == 1.0, data_last_selection[_ac_coln], np.nan
  )

  sig_ac_offset_value = 0


  # Make Buy Signal plot
  acb_plot = mpf.make_addplot(
      data_last_selection[_acb_coln] + sig_ac_offset_value,
      panel=ac_plot_panel_id,
      type="scatter",
      markersize=ac_signals_marker_size,
      marker=ac_signal_marker,
      color=ac_signal_buy_color,
  )
  # Make Sell Signal plot
  acs_plot = mpf.make_addplot(
      data_last_selection[_acs_coln] - sig_ac_offset_value,
      panel=ac_plot_panel_id,
      type="scatter",
      markersize=ac_signals_marker_size,
      marker=ac_signal_marker,
      color=ac_signal_sell_color,
  )


  #%% Print the summary statistics of the 'ac' column
  l.debug("Summary Stats:")
  l.debug(data_last_selection[_ac_coln].describe())



  #%% Make FDB plot


  fdb_tick_offset = average_bar_height  # pipsize * 111
  fdb_offset_value = average_bar_height / 2  # pipsize * fdb_tick_offset

  fdbb_up_plot, fdbs_down_plot = make_plot__fdb_signals(fdb_signal_buy_color, fdb_signal_sell_color, fdb_marker_size, fdb_signal_marker, _fdbb_coln, _fdbs_coln, main_plot_panel_id, data_last_selection, fdb_offset_value)


  #%% Make Fractals plot


  fractal_offset_value = average_bar_height / 2  # pipsize * fractal_tick_offset

  fractal_tick_offset = pipsize * 111  # price_mean*10

  # Align fractals with OHLC bars


  data_last_selection.loc[:,fh_col_dim] = np.where(
      (data_last_selection[_high_coln] > data_last_selection[_teeth_coln])
      & (data_last_selection[fh_col_dim] == True),
      data_last_selection[_high_coln],
      np.nan,
  )


  data_last_selection.loc[:,fl_col_dim] = np.where(
      (data_last_selection[_low_coln] < data_last_selection[_teeth_coln])
      & (data_last_selection[fl_col_dim] == True),
      data_last_selection[_low_coln] - fractal_offset_value,
      np.nan,
  )





  fractal_up_plot, fractal_down_plot = make_plot__fractals_indicator(fractal_up_color, fractal_dn_color, fractal_marker_size, fractal_up_marker, fractal_dn_marker, fh_col_dim, fl_col_dim, main_plot_panel_id, data_last_selection, fractal_offset_value)

  #%% Fractal higher dim


  data_last_selection.loc[:,fh_col_dim_higher] = np.where(
      (data_last_selection[_high_coln] > data_last_selection[_teeth_coln])
      & (data_last_selection[fh_col_dim_higher] == True),
      data_last_selection[_high_coln],
      np.nan,
  )


  data_last_selection.loc[:,fl_col_dim_higher] = np.where(
      (data_last_selection[_low_coln] < data_last_selection[_teeth_coln])
      & (data_last_selection[fl_col_dim_higher] == True),
      data_last_selection[_low_coln] - fractal_offset_value,
      np.nan,
  )


  #%% PLot higher fractal


  fractal_up_plot_higher, fractal_down_plot_higher = make_plot_fractals_degreehigher_indicator(fractal_dn_color_higher, fractal_up_color_higher, fractal_degreehigher_marker_size, fractal_up_marker_higher, fractal_dn_marker_higher, fh_col_dim_higher, fl_col_dim_higher, main_plot_panel_id, data_last_selection, fractal_offset_value)





  #%% Print Mean
  #l.debug("Mean: " + (price_mean).astype(str))


  #%% Plotting
  addplot = [
      jaw_plot,
      teeth_plot,
      lips_plot,
      fractal_up_plot,
      fractal_down_plot,
      fractal_up_plot_higher,
      fractal_down_plot_higher,
      fdbb_up_plot,
      fdbs_down_plot,
      sb_plot,
      ss_plot,
      ao_plot,
      ac_plot,
      acs_plot,
      acb_plot,
  ]

  # Filter out empty plots
  #addplot = [plot for plot in addplot if plot['data'].size > 0]

  fig, axes = mpf.plot(
      ohlc,
      type=main_plot_type,
      style=plot_style,
      addplot=addplot,
      volume=False,
      figratio=(fig_ratio_x, fig_ratio_y),
      title=instrument + "  " + timeframe,
      returnfig=True,
      tight_layout=True,
  )

  # fig, axes = mpf.plot(
  #     ohlc,
  #     type=main_plot_type,
  #     style=plot_style,
  #     addplot=[
  #         jaw_plot,
  #         teeth_plot,
  #         lips_plot,
  #         fractal_up_plot,
  #         fractal_down_plot,
  #         fractal_up_plot_higher,
  #         fractal_down_plot_higher,
  #         fdbb_up_plot,
  #         fdbs_down_plot,
  #         sb_plot,
  #         ss_plot,
  #         ao_plot,
  #         ac_plot,
  #         acs_plot,
  #         acb_plot,
  #     ],
  #     volume=False,
  #     figratio=(fig_ratio_x, fig_ratio_y),
  #     title=instrument + "  " + timeframe,
  #     returnfig=True,
  #     tight_layout=True,
  # )

  # Set y-axis limits
  axes[main_plot_panel_id].set_ylim(
      low_min - fdb_offset_value - pipsize * 20,
      high_max + fdb_offset_value + pipsize * 20,
  )

  # Get current x-axis limits
  x_min, x_max = axes[main_plot_panel_id].get_xlim()

  axe2ymin, axe2ymax = axes[ac_plot_panel_id].get_ylim()
  l.debug("axe2ymin: " + str(axe2ymin))
  l.debug("axe2ymax: " + str(axe2ymax))

  # Calculate new x-axis limit
  new_x_max = x_max + 8  # Add 8 for future bars

  # Set new x-axis limit
  axes[main_plot_panel_id].set_xlim(x_min, new_x_max)


  # Align the title to the left
  fig.suptitle(instrument + "  " + timeframe, x=0.05, ha="left")

  # Set the font size of the x-axis labels
  for ax in axes:
      ax.tick_params(axis="x", labelsize=6)

  # Set the font size of the Date column
  axes[main_plot_panel_id].tick_params(axis="x", labelsize=6)

  if show_plot:
      plt.show()
  return fig,axes

def make_plot_fractals_degreehigher_indicator(fractal_dn_color_higher, fractal_up_color_higher, fractal_degreehigher_marker_size, fractal_up_marker_higher, fractal_dn_marker_higher, fh_col_dim_higher, fl_col_dim_higher, main_plot_panel_id, data_last_selection, fractal_offset_value):
    fractal_up_plot_higher = mpf.make_addplot(
      data_last_selection[fh_col_dim_higher] + fractal_offset_value,
      panel=main_plot_panel_id,
      type="scatter",
      markersize=fractal_degreehigher_marker_size,
      marker=fractal_up_marker_higher,
      color=fractal_up_color_higher,
  )
    fractal_down_plot_higher = mpf.make_addplot(
      data_last_selection[fl_col_dim_higher],
      panel=main_plot_panel_id,
      type="scatter",
      markersize=fractal_degreehigher_marker_size,
      marker=fractal_dn_marker_higher,
      color=fractal_dn_color_higher,
  )
    
    return fractal_up_plot_higher,fractal_down_plot_higher

def make_plot__fractals_indicator(fractal_up_color, fractal_dn_color, fractal_marker_size, fractal_up_marker, fractal_dn_marker, fh_col_dim, fl_col_dim, main_plot_panel_id, data_last_selection, fractal_offset_value):
    fractal_up_plot = mpf.make_addplot(
      data_last_selection[fh_col_dim] + fractal_offset_value,
      panel=main_plot_panel_id,
      type="scatter",
      markersize=fractal_marker_size,
      marker=fractal_up_marker,
      color=fractal_up_color,
  )
    fractal_down_plot = mpf.make_addplot(
      data_last_selection[fl_col_dim],
      panel=main_plot_panel_id,
      type="scatter",
      markersize=fractal_marker_size,
      marker=fractal_dn_marker,
      color=fractal_dn_color,
  )
    
    return fractal_up_plot,fractal_down_plot


def make_plot__fdb_signals(fdb_signal_buy_color, fdb_signal_sell_color, fdb_marker_size, fdb_signal_marker, fdbb_coln, fdbs_coln, main_plot_panel_id, data_last_selection, fdb_offset_value):
        """
        Creates scatter plots for FDB buy and sell signals based on the given parameters.

        Args:
                fdb_signal_buy_color (str): Color of the scatter plot for buy signals.
                fdb_signal_sell_color (str): Color of the scatter plot for sell signals.
                fdb_marker_size (int): Size of the markers in the scatter plot.
                fdb_signal_marker (str): Marker style for the scatter plot.
                fdbb_coln (str): Column name for buy signals in the data.
                fdbs_coln (str): Column name for sell signals in the data.
                main_plot_panel_id (int): ID of the main plot panel.
                data_last_selection (pandas.DataFrame): Data containing the buy and sell signals.
                fdb_offset_value (float): Offset value to adjust the scatter plot positions.

        Returns:
                tuple: A tuple containing the scatter plot for buy signals and the scatter plot for sell signals.
        """
        
        fdbb_up_plot = make_plot_fdbb_signal(fdb_signal_buy_color, fdb_marker_size, fdb_signal_marker, fdbb_coln, main_plot_panel_id, data_last_selection, fdb_offset_value)

        fdbs_down_plot = make_plot_fdbs_signal(fdb_signal_sell_color, fdb_marker_size, fdb_signal_marker, fdbs_coln, main_plot_panel_id, data_last_selection, fdb_offset_value)
        
        return fdbb_up_plot,fdbs_down_plot

def make_plot_fdbs_signal(fdb_signal_sell_color, fdb_marker_size, fdb_signal_marker, fdbs_coln, main_plot_panel_id, data_last_selection, fdb_offset_value):
    fdbs_down_plot = mpf.make_addplot(
            data_last_selection[fdbs_coln] - fdb_offset_value,
            panel=main_plot_panel_id,
            type="scatter",
            markersize=fdb_marker_size,
            marker=fdb_signal_marker,
            color=fdb_signal_sell_color,
    )

    return fdbs_down_plot

def make_plot_fdbb_signal(fdb_signal_buy_color, fdb_marker_size, fdb_signal_marker, fdbb_coln, main_plot_panel_id, data_last_selection, fdb_offset_value):
    fdbb_up_plot = mpf.make_addplot(
            data_last_selection[fdbb_coln] + fdb_offset_value,
            panel=main_plot_panel_id,
            type="scatter",
            markersize=fdb_marker_size,
            marker=fdb_signal_marker,
            color=fdb_signal_buy_color,
    )

    return fdbb_up_plot
  #return plt


# %% ALias function (future name)

def plotcdf(data,instrument, timeframe, nb_bar_on_chart=375,show_plot=True):
  return plot_from_cds_df(data,instrument,timeframe,nb_bar_on_chart,show_plot)

def plot(instrument, timeframe, nb_bar_on_chart=375, recreate_data=True, show_plot=True):
    """
    Plot the chart for a given instrument and timeframe.

    Parameters:
    instrument (str): The name of the instrument.
    timeframe (str): The timeframe for the chart.
    nb_bar_on_chart (int, optional): The number of bars to display on the chart. Default is 375.
    recreate_data (bool, optional): Whether to recreate the data for the chart. Default is True.
    show_plot (bool, optional): Whether to display the plot. Default is True.

    Returns:
    fig: The figure object of the plot.
    axes: The axes object of the plot.
    """
    fig, axes = jgtxplot18c_231209(instrument, timeframe, nb_bar_on_chart, recreate_data, show_plot)
    return fig, axes

  









#@STCGoal Aim to become the container for lighting JGTADS

import sys
import os
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))
import JGTPDSP as pds
import JGTIDS as ids
import JGTCDS as cds
import jgtwslhelper as wsl


import logging
_loglevel= logging.WARNING

# Create a logger object
l = logging.getLogger()
l.setLevel(_loglevel)

# Create a console handler and set its level
console_handler = logging.StreamHandler()
console_handler.setLevel(_loglevel)

# Create a formatter and add it to the console handler
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
console_handler.setFormatter(formatter)

# Add the console handler to the logger
l.addHandler(console_handler)





def prepare_cds_for_ads_data(instrument, timeframe, nb_bar_on_chart, recreate_data=True):
    """
    Prepare CDS (Credit Default Swap) data for ADS (Automated Trading System).

    Args:
        instrument (str): The instrument symbol.
        timeframe (str): The timeframe of the data.
        nb_bar_on_chart (int): The number of bars to display on the chart.
        recreate_data (bool, optional): Whether to recreate the data. Defaults to True.

    Returns:
        pandas.DataFrame: The prepared CDS data.

    """
    cache_data=False
    cache_dir = "cache"
    if cache_data:
        os.makedirs(cache_dir, exist_ok=True)

    fn =  instrument.replace("/", "-") + "_" + timeframe + ".csv"
    fnpath = os.path.join(cache_dir,fn)
    l.info("fnpath:"+ fnpath)

    #%% Load data
    l.info("-----------------  CDS  -----------------")
    if recreate_data:
        try:
            df = pds.getPH(instrument,timeframe,nb_bar_on_chart)
        except:
            l.warning("Could not get DF, trying to run thru WSL the update")
            wsl.jgtfxcli(instrument, timeframe, nb_bar_on_chart+35)
            df = pds.getPH(instrument,timeframe,nb_bar_on_chart)
        # Select the last 400 bars of the data
        try:
            selected = df.iloc[-nb_bar_on_chart-120:].copy()
        except:
            selected = df.copy()
            l.warning("Could not select the desired amount of bars, trying anyway with what we have")
            pass
        #print(selected)
        data = cds.createFromDF(selected)
        if cache_data:
            data.to_csv(fnpath)
    return data


def get(instrument= "EUR/USD",timeframe="H4",nb_bar_on_chart=1000):
  data = prepare_cds_for_ads_data(instrument, timeframe, nb_bar_on_chart)
  return data
#p=pds.getPH(instrument,timeframe)





 # OrderMonitor.py
# Copyright 2019 Gehtsoft USA LLC

# Licensed under the license derived from the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at

# http://fxcodebase.com/licenses/open-source/license.html

# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List
from enum import Enum
import json
import os
import tlid

#------------------------#

# common.py


import logging
import datetime
import traceback
import argparse
import sys


try :
    import __main__
    logging.basicConfig(filename='{0}.log'.format(__main__.__file__), level=logging.INFO,
                    format='%(asctime)s %(levelname)s %(message)s', datefmt='%m.%d.%Y %H:%M:%S')
    console = logging.StreamHandler(sys.stdout)
    console.setLevel(logging.INFO)
    logging.getLogger('').addHandler(console)
except:
    #print('logging failed - dont worry')
    pass

def add_main_arguments(parser: argparse.ArgumentParser):
    parser.add_argument('--login',
                        metavar="LOGIN",
                        required=True,
                        help='Your user name.')

    parser.add_argument('--password',
                        metavar="PASSWORD",
                        required=True,
                        help='Your password.')

    parser.add_argument('--urlserver',
                        metavar="URL",
                        required=True,
                        help='The server URL. For example,\
                                 https://www.fxcorporate.com/Hosts.jsp.')

    parser.add_argument('--connection',
                        metavar="CONNECTION",
                        required=True,
                        help='The connection name. For example, \
                                 "Demo" or "Real".')


    parser.add_argument('-session',
                        help='The database name. Required only for users who\
                                 have accounts in more than one database.\
                                 Optional parameter.')

    parser.add_argument('-pin',
                        help='Your pin code. Required only for users who have \
                                 a pin. Optional parameter.')

def add_candle_open_price_mode_argument(parser: argparse.ArgumentParser):
    parser.add_argument('--openpricemode',
                        metavar="CANDLE_OPEN_PRICE_MODE",
                        default="prev_close",
                        help='Ability to set the open price candles mode. \
                        Possible values are first_tick, prev_close. For more information see description \
                        of O2GCandleOpenPriceMode enumeration. Optional parameter.')

def add_instrument_timeframe_arguments(parser: argparse.ArgumentParser, timeframe: bool = True):
    parser.add_argument('-i','--instrument',
                        metavar="INSTRUMENT",
                        default="EUR/USD",
                        help='An instrument which you want to use in sample. \
                                  For example, "EUR/USD".')

    if timeframe:
        parser.add_argument('-t','--timeframe',
                            metavar="TIMEFRAME",
                            default="m5",
                            help='Time period which forms a single candle. \
                                      For example, m1 - for 1 minute, H1 - for 1 hour.')
    parser.add_argument('-ip',
                        metavar="IndicatorPattern",
                        required=False,
                        help='The indicator Pattern. For example, \
                                 "AOAC","JTL,"JTLAOAC","JTLAOAC","AOACMFI".')

def add_direction_rate_lots_arguments(parser: argparse.ArgumentParser, direction: bool = True, rate: bool = True,
                                      lots: bool = True):
    if direction:
        parser.add_argument('-d', metavar="TYPE", required=True,
                            help='The order direction. Possible values are: B - buy, S - sell.')
    if rate:
        parser.add_argument('-r', metavar="RATE", required=True, type=float,
                            help='Desired price of an entry order.')
    if lots:
        parser.add_argument('-lots', metavar="LOTS", default=1, type=int,
                            help='Trade amount in lots.')


def add_account_arguments(parser: argparse.ArgumentParser):
    parser.add_argument('-account', metavar="ACCOUNT",
                        help='An account which you want to use in sample.')


def str_to_datetime(date_str):
    formats = ['%m.%d.%Y %H:%M:%S', '%Y-%m-%d %H:%M:%S', '%Y/%m/%d %H:%M:%S', '%Y/%m/%d', '%Y-%m-%d']
    
    for fmt in formats:
        try:
            return datetime.datetime.strptime(date_str, fmt)
        except ValueError:
            continue
    return None

def tlid_range_to_start_end_datetime(tlid_range: str):
    start_str, end_str = tlid_range.split("_")
    
    date_format_start = "%y%m%d%H%M"
    date_format_end = "%y%m%d%H%M"
    
    if len(start_str) == 6:
        date_format_start = "%y%m%d"
    if len(end_str) == 6:
        date_format_end = "%y%m%d"
    
    #print(date_format_end)
    try:
        start_dt =  datetime.datetime.strptime(start_str, date_format_start)
        end_dt = datetime.datetime.strptime(end_str, date_format_end)
        return start_dt,end_dt
    except ValueError:
        return None

def tlid_range_to_jgtfxcon_start_end_str(tlid_range: str):
    date_format_fxcon = '%m.%d.%Y %H:%M:%S'
    start_dt,end_dt = tlid_range_to_start_end_datetime(tlid_range)
    #print(str(start_dt),str(end_dt))
    if start_dt is None or end_dt is None:
        return None,None
    else:
        return str(start_dt.strftime(date_format_fxcon)),str(end_dt.strftime(date_format_fxcon))

def tlid_dt_to_string(dt):
    return dt.strftime("%y%m%d%H%M")

def tlidmin_to_dt(tlid_str: str):
    date_format = "%y%m%d%H%M"
    try:
        tlid_dt =  datetime.datetime.strptime(tlid_str, date_format)
        return tlid_dt
    except ValueError:
        pass
    
    return None

def valid_datetime(check_future: bool):
    def _valid_datetime(str_datetime: str):
        date_format = '%m.%d.%Y %H:%M:%S'
        try:
            result = datetime.datetime.strptime(str_datetime, date_format).replace(
                tzinfo=datetime.timezone.utc)
            if check_future and result > datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc):
                msg = "'{0}' is in the future".format(str_datetime)
                raise argparse.ArgumentTypeError(msg)
            return result
        except ValueError:
            now = datetime.datetime.now()
            msg = "The date '{0}' is invalid. The valid data format is '{1}'. Example: '{2}'".format(
                str_datetime, date_format, now.strftime(date_format))
            raise argparse.ArgumentTypeError(msg)
    return _valid_datetime


def add_tlid_range_argument(parser: argparse.ArgumentParser):
    #print("Tlid range active")
    parser.add_argument('-r', '--range', type=str, required=False, dest='tlidrange',
                        help='TLID range in the format YYMMDDHHMM_YYMMDDHHMM.')

def add_date_arguments(parser: argparse.ArgumentParser, date_from: bool = True, date_to: bool = True):
    if date_from:
        parser.add_argument('-s','--datefrom',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Date/time from which you want to receive\
                                      historical prices. If you leave this argument as it \
                                      is, it will mean from last trading day. Format is \
                                      "m.d.Y H:M:S". Optional parameter.',
                            type=valid_datetime(True)
                            )
    if date_to:
        parser.add_argument('-e','--dateto',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Datetime until which you want to receive \
                                      historical prices. If you leave this argument as it is, \
                                      it will mean to now. Format is "m.d.Y H:M:S". \
                                      Optional parameter.',
                            type=valid_datetime(False)
                            )


def add_report_date_arguments(parser: argparse.ArgumentParser, date_from: bool = True, date_to: bool = True):
    if date_from:
        parser.add_argument('-s','--datefrom',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Datetime from which you want to receive\
                                      combo account statement report. If you leave this argument as it \
                                      is, it will mean from last month. Format is \
                                      "m.d.Y H:M:S". Optional parameter.',
                            type=valid_datetime(True)
                            )
    if date_to:
        parser.add_argument('-e','--dateto',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Datetime until which you want to receive \
                                      combo account statement report. If you leave this argument as it is, \
                                      it will mean to now. Format is "m.d.Y H:M:S". \
                                      Optional parameter.',
                            type=valid_datetime(True)
                            )


def add_max_bars_arguments(parser: argparse.ArgumentParser):
    parser.add_argument('-c','--quotescount',
                        metavar="MAX",
                        default=335,
                        type=int,
                        help='Max number of bars. 0 - Not limited')


# def add_bars_arguments(parser: argparse.ArgumentParser):
#     parser.add_argument('-bars',
#                         metavar="COUNT",
#                         default=3,
#                         type=int,
#                         help='Build COUNT bars. Optional parameter.')


def add_output_argument(parser: argparse.ArgumentParser):
    """
    Adds an output argument to the given argument parser.

    Args:
        parser (argparse.ArgumentParser): The argument parser to add the output argument to.

    Returns:
        None
    """
    parser.add_argument('-o','--output',
                        action='store_true',
                        help='Output file. If specified, output will be written in the filestore.')
    
    parser.add_argument('-z','--compress',
                        action='store_true',
                        help='Compress the output. If specified, it will also activate the output flag.')

    return parser


# def add_quiet_argument(parser):
#     parser.add_argument('-q','--quiet',
#                         action='store_true',
#                         help='Suppress all output. If specified, no output will be printed to the console.')
#     return parser

def add_verbose_argument(parser):
    parser.add_argument('-v', '--verbose',
                        type=int,
                        default=0,
                        help='Set the verbosity level. 0 = quiet, 1 = normal, 2 = verbose, 3 = very verbose, etc.')
    return parser

def add_cds_argument(parser):
    parser.add_argument('-cds','--cds',
                        action='store_true',
                        default=False,
                        help='Action the creation of CDS')
    return parser

def add_iprop_init_argument(parser):
    parser.add_argument('-iprop','--iprop',
                        action='store_true',
                        default=False,
                        help='Toggle the downloads of all instrument properties ')
    return parser

def add_debug_argument(parser):
    parser.add_argument('-debug','--debug',
                        action='store_true',
                        default=False,
                        help='Toggle debug ')
    return parser

def add_pdsserver_argument(parser):
    parser.add_argument('-server','--server',
                        action='store_true',
                        default=False,
                        help='Run the server ')
    return parser


def print_exception(exception: Exception):
    logging.error("Exception: {0}\n{1}".format(exception, traceback.format_exc()))











def diff_month(year: int, month: int, date2: datetime):
    return (year - date2.year) * 12 + month - date2.month






_JGT_CONFIG_JSON_SECRET=None

def readconfig(json_config_str=None):
    global _JGT_CONFIG_JSON_SECRET
    # Try reading config file from current directory

    if json_config_str is not None:
        config = json.loads(json_config_str)
        _JGT_CONFIG_JSON_SECRET=json_config_str
        return config


    if _JGT_CONFIG_JSON_SECRET is not None:
        config = json.loads(_JGT_CONFIG_JSON_SECRET)
        return config
    
    # Otherwise, try reading config file from current directory, home or env var
    config_file = 'config.json'
    config = None

    if os.path.isfile(config_file):
        with open(config_file, 'r') as f:
            config = json.load(f)
            return config
    else:
        # If config file not found, check home directory
        home_dir = os.path.expanduser("~")
        config_file = os.path.join(home_dir, 'config.json')
        if os.path.isfile(config_file):
            with open(config_file, 'r') as f:
                config = json.load(f)
        else:
            # If config file still not found, try reading from environment variable
            config_json_str = os.getenv('JGT_CONFIG_JSON_SECRET')
            if config_json_str:
                config = json.loads(config_json_str)
                return config


    # Now you can use the config dictionary in your application

    # Read config file
    with open(config_file, 'r') as file:
        config = json.load(file)
        
    if config is None:
        raise Exception("Configuration not found")
    
    return config



#!/usr/bin/env Python
"""
jgtpy
"""

from setuptools import find_packages, setup

from jgtpy import __version__ as version

INSTALL_REQUIRES = [
    'pandas>=0.25.1',
    'python-dotenv>=0.19.2',
    #'kaleido>=0.2.1',
    'matplotlib>=3.3.1',
    'plotly>=5.18.0',
    'jgtapy>=1.9.12',
    'dropbox>=11.36.2',
    'mplfinance>=0.12.10b0',    
	'tlid>=0.0.6',
]

EXTRAS_DEV_LINT = [
    "flake8>=3.6.0,<3.7.0",
    "isort>=4.3.4,<4.4.0",
]

EXTRAS_DEV_TEST = [
    "coverage",
    "pytest>=3.10",
]

EXTRAS_DEV_DOCS = [
    "readme_renderer",
    "sphinx",
    "sphinx_rtd_theme>=0.4.0",
    "html2text>=2020.1.16",
    "html2markdown>=0.1.7",
]

setup(
    name='jgtpy',
    version=version,
    description='JGTrading Data maker\' Dataframes',
    long_description=open('README.rst').read(),
    author='GUillaume Isabelle',
    author_email='jgi@jgwill.com',
    url='https://github.com/jgwill/jgtpy',
    packages=find_packages(include=['jgtpy'], exclude=['*test*']),
    # packages=find_packages(include=['jgtpy', 'jgtpy.forexconnect', 'jgtpy.forexconnect.lib', 'jgtpy.forexconnect.lib.windows', 'jgtpy.forexconnect.lib.linux'], exclude=['*test*']),
    install_requires=INSTALL_REQUIRES,
    entry_points={
        'console_scripts': ['jgtcli=jgtpy.jgtcli:main'],
    },
    extras_require={
        'dev': (EXTRAS_DEV_LINT + EXTRAS_DEV_TEST + EXTRAS_DEV_DOCS),
        'dev-lint': EXTRAS_DEV_LINT,
        'dev-test': EXTRAS_DEV_TEST,
        'dev-docs': EXTRAS_DEV_DOCS,
    },
    license='MIT',
    keywords='data',
    classifiers=[
        "Development Status :: 5 - Production/Stable", 
        "Intended Audience :: Financial and Insurance Industry",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent", 
        "Topic :: Software Development :: Libraries :: Python Modules", 
        "Programming Language :: Python :: 3.7.16", 
    ],
)
